__version__ = '1.0.3'

from .importer_csv import CEImporter_CSV
from .importer_pdf import CEImporter_PDF

from beancount.ingest import importer


class CEImporter(importer.ImporterProtocol):
    """Beancount Importer for Caisse d'Epargne PDF and CSV statement.

    Attributes:
        iban (str): International Bank Account Number of the account you want to extract operations. Note that only the account number is necessary.
        account (str): Account name in beancount format (e.g. 'Assets:FR:CdE:CompteCourant')
        file_type (int): Define file type treated by this importer instance: 0 -> PDF and CSV, 1 -> only PDF, 2 -> only CSV. Defaults to PDF and CSV.
        expenseCat (str, optional): Expense category in beancount format (e.g. 'Expenses:FIXME'). Defaults to '', no expense posting added to the operation.
        creditCat (str, optional): Income category in beancount format (e.g. 'Income:FIXME'). Defaults to '', no income posting added to the operation.
        showOperationTypes (bool, optional): Show or not operation type (CARDDEBIT, WIRETRANSFER, CHECK ...) in header. Only possible for PDF statements. Defaults to False.
    """

    def __init__(
        self,
        iban: str,
        account: str,
        file_type: int = 0,
        expenseCat: str = '',
        creditCat: str = '',
        showOperationTypes: bool = False,
    ):
        self.iban = iban
        self.account = account
        self.expenseCat = expenseCat
        self.creditCat = creditCat
        self.showOperationTypes = showOperationTypes
        self.file_type = file_type

        assert self.file_type in [0, 1, 2]
        if self.file_type in [0, 2]:
            self.csv_importer = CEImporter_CSV(
                account=self.account,
                expenseCat=self.expenseCat,
                creditCat=self.creditCat,
                iban=self.iban,
            )
        if self.file_type in [0, 1]:
            self.pdf_importer = CEImporter_PDF(
                iban=self.iban,
                account=self.account,
                expenseCat=self.expenseCat,
                creditCat=self.creditCat,
                showOperationTypes=self.showOperationTypes,
            )

    def _get_importer(self, file_):
        ext_str = file_.name.split('.')[-1].lower()
        if self.file_type == 0:
            if self.pdf_importer.identify(file_):
                return self.pdf_importer
            if self.csv_importer.identify(file_):
                return self.csv_importer
        if (self.file_type == 1) and self.pdf_importer.identify(file_):
            return self.pdf_importer
        if (self.file_type == 2) and self.csv_importer.identify(file_):
            return self.csv_importer
        return None

    ## API Methods ##
    #################

    def name(self):
        return 'Caisse Epargne: {}'.format(self.__class__.__name__)

    def file_account(self, file_):
        importer = self._get_importer(file_)
        return importer.file_account(file_) if importer else None

    def file_date(self, file_):
        importer = self._get_importer(file_)
        return importer.file_date(file_) if importer else None

    def file_name(self, file_):
        importer = self._get_importer(file_)
        return importer.file_name(file_) if importer else None

    def identify(self, file_) -> bool:
        if self.file_type == 0:
            return self.pdf_importer.identify(
                file_
            ) or self.csv_importer.identify(file_)
        if self.file_type == 1:
            return self.pdf_importer.identify(file_)
        if self.file_type == 2:
            return self.csv_importer.identify(file_)
        return False

    def extract(self, file_, existing_entries=None):
        importer = self._get_importer(file_)
        return importer.extract(file_, existing_entries) if importer else None



import csv
from datetime import datetime
from typing import Mapping, Tuple

from beancount.core import data
from beancount.core.amount import Amount
from beancount.core.number import Decimal
from beancount.ingest import importer


class CEImporter_CSV(importer.ImporterProtocol):
    """Beancount Importer for Caisse d'Epargne CSV statement exports.

    Attributes:
        account (str): Account name in beancount format (e.g. 'Assets:FR:CdE:CompteCourant')
        expenseCat (str, optional): Expense category in beancount format (e.g. 'Expenses:FIXME'). Defaults to '', no expense posting added to the operation.
        creditCat (str, optional): Income category in beancount format (e.g. 'Income:FIXME'). Defaults to '', no income posting added to the operation.
        iban (str, optional): International Bank Account Number of the account you want to extract operations. Note that only the account number is necessary. File will be skipped if account numbers do not match.
    """

    def __init__(
        self,
        account: str,
        expenseCat: str = '',
        creditCat: str = '',
        iban: str = '',
    ):
        self.account = account
        self.expenseCat = expenseCat
        self.creditCat = creditCat
        self.iban = iban

    ## API Methods ##
    #################

    def name(self):
        return 'Caisse Epargne: {}'.format(self.__class__.__name__)

    def file_account(self, _):
        return self.account

    def file_date(self, file_):
        if not self.identify(file_):
            return None
        date = None
        with open(file_.name) as fd:
            # for _ in range(4):
            #     next(fd)
            reader = csv.DictReader(
                fd, delimiter=';', quoting=csv.QUOTE_MINIMAL, quotechar='"'
            )
            for line in reader:
                try:
                    date_tmp = datetime.strptime(
                        line["Date operation"], '%Y-%m-%d'
                    ).date()
                except Exception as e:
                    print(e)
                    break
                if not date or date_tmp > date:
                    date = date_tmp
        return date

    def file_name(self, _):
        return 'CaisseEpargne_Statement.csv'

    def is_valid_header(self, line: str) -> bool:
        # expected_values = [
        #     "Date",
        #     "Numéro d'opération",
        #     "Libellé",
        #     "Débit",
        #     "Crédit",
        #     "Détail",
        # ]
        expected_values = [
            "Date operation",
            "Date de comptabilisation",
            "Categorie operation",
            "Libelle operation",
            "Libelle simplifie",
            "Montant operation",
            "Pointage operation",
        ]
        actual_values = [column.strip('\n') for column in line.split(';')]
        for (expected, actual) in zip(expected_values, actual_values):
            if expected != actual:
                return False
        return True

    def is_valid_account_number_line(self, line: str) -> bool:
        if not self.iban:
            return True
        try:
            [account_number_key, account_number] = [
                token.strip() for token in line.split(';')[0].split(':')
            ]
            return account_number_key == "Numéro de compte" and account_number.replace(
                ' ', ''
            ) in self.iban.replace(
                ' ', ''
            )
        except:
            return False

    def identify(self, file_) -> bool:
        try:
            with open(file_.name) as fd:
                # next(fd)
                # account_number_line = next(fd)
                # next(fd)
                # next(fd)
                header_line = next(fd)
            return self.is_valid_header(
                header_line
            )  # and self.is_valid_account_number_line(account_number_line)
        except:
            return False

    def extract(self, file_, existing_entries=None):
        entries = []

        if not self.identify(file_):
            return []

        with open(file_.name) as fd:
            # for _ in range(4):
            #     next(fd)
            reader = csv.DictReader(
                fd, delimiter=';', quoting=csv.QUOTE_MINIMAL, quotechar='"'
            )
            for index, line in enumerate(reader):
                meta = data.new_metadata(file_.name, index)
                postings = []
                try:
                    date = datetime.strptime(
                        line["Date operation"], '%Y-%m-%d'
                    ).date()
                except:
                    break
                amount = Decimal(line["Montant operation"].replace(',', '.'))
                postings.append(
                    data.Posting(
                        self.account,
                        Amount(amount, 'EUR'),
                        None,
                        None,
                        None,
                        None,
                    )
                )
                if amount < 0:
                    if len(self.expenseCat) > 0:
                        postings.append(
                            data.Posting(
                                self.expenseCat,
                                Amount(-amount, 'EUR'),
                                None,
                                None,
                                None,
                                None,
                            )
                        )
                else:
                    if len(self.creditCat) > 0:
                        postings.append(
                            data.Posting(
                                self.creditCat,
                                Amount(-amount, 'EUR'),
                                None,
                                None,
                                None,
                                None,
                            )
                        )
                entries.append(
                    data.Transaction(
                        meta,
                        date,
                        self.FLAG,
                        line["Libelle operation"]
                        + " ["
                        + line["Libelle simplifie"]
                        + "]",
                        '',
                        data.EMPTY_SET,
                        data.EMPTY_SET,
                        postings,
                    )
                )
        return entries



# - will match owner
# prior to march 2019
owner_regex_v1 = r'Identifiant client\s+(?P<owner>\D*)'
# after march 2019
owner_regex_v2 = r'^(?P<title>MR|MME|MLLE)\s+(?P<owner>\D*?)$'

# - will match dates
emission_date_regex = r'\b(?P<date>[\d/]{10})\b'

# - will match debits
# Ex 1.
# 18/10 CB CENTRE LECLERC  FACT 161014      13,40
# Ex 2.
# 27/05 PRLV FREE MOBILE      3,99
# -Réf. donneur d'ordre :
# fmpmt-XXXXXXXX
# -Réf. du mandat : FM-XXXXXXXX-X
# [\S\s].*?
debit_regex = (
    r'^'
    # date: dd/dd
    r'(?P<op_dte>\d\d\/\d\d)'
    # label: any single character (.), between 0 and unlimited (*), lazy (?)
    r'(?P<op_lbl>.*?)'
    # any whitespace and non-whitespace character (i.e. any character) ([\S\s]), any character (.) between 0 and unlimited (+), lazy
    r'\s.*?'
    # amount: alternative between ddd ddd,dd and ddd,dd, until the end of line ($)
    #    the positive lookebehind assures that there is at least one white space before any amount
    #    the positive lookbehind handles the following case where amount to match is 4,45 and not 14,40:
    #    19/10 INTERETS TAEG 14,40
    #    VALEUR AU 18/10     4,45
    r'(?P<op_amt>(?<=\s)\d{1,3}\s{1}\d{1,3}\,\d{2}|\d{1,3}\,\d{2}(?!([\S\s].*?((?<=(?=(^(?!(?1))\s.*(?1))))\s.*(?3)))))$'
    # any whitespace character (\s), between 0 and unlimited (*), greedy
    r'\s*'
    # extra label: 'single line mode' until the positive lookehead is satisfied
    #    positive lookahead --> alternative between:
    #       -line starting with first named subpatern (date)
    #       -line starting with third named subpatern (amount)
    #       -EOL
    #    we use [\s\S]*? to do like the single line mode
    #    basically it's going to match any non-whitespace OR whitespace character. That is, any character, including linebreaks.
    #    we could have used (?s) to activate the real line mode...
    #    ...but Python doesn't support mode-modified groups (meaning that it will change the mode for the whole regex)
    r'(?P<op_lbl_extra>[\S\s]*?(?=^(?1)|^(?3)|\Z))'
)

# - will match credits
# Ex 1.
# 150,0008/11 VIREMENT PAR INTERNET
# Ex 2.
# 11,8011/02VIR SEPA LA MUTUELLE DES ETUDIA
# XXXXX/XX/XX-XXXX/XXXXXXXXX
# -Réf. donneur d'ordre :
# XXXXX/XX/XX-XXXX/XXXXXXXXX
credit_regex = (
    r'^'
    ## amount: alternative between ddd ddd,dd and ddd,dd
    r'(?P<op_amt>\d{1,3}\s{1}\d{1,3}\,\d{2}|\d{1,3}\,\d{2})'
    ## date: dd/dd
    r'(?P<op_dte>\d\d\/\d\d)'
    r'(?P<op_lbl>.*)$'
    ## any whitespace character (\s), between 0 and unlimited (*), greedy
    r'\s*'
    ## extra label: 'single line mode' until the positive lookehead is satisfied
    #    positive lookahead --> alternative between:
    #      -line starting with first subpatern (amount)
    #      -line starting with second subpatern (date)
    #      -EOL
    #    we use [\s\S]*? to do like the single line mode
    #    basically it's going to match any non-whitespace OR whitespace character. That is, any character, including linebreaks.
    #    we could have used (?s) to activate the real line mode...
    #    ...but Python doesn't support mode-modified groups (meaning that it will change the mode for the whole regex)
    r'(?P<op_lbl_extra>[\S\s]*?(?=^(?1)|^(?2)|\Z))'
)

# - will match new account balances
#   NOUVEAU SOLDE CREDITEUR AU 15/11/14 (en francs : 1 026,44) 156,48
new_balance_regex = r'NOUVEAU SOLDE CREDITEUR AU (?P<bal_dte>\d\d\/\d\d\/\d\d)\s+\(en francs : (?P<bal_amt_fr>[\d, ]+)\)\s+(?P<bal_amt>[\d, ]+?)$'

one_character_line_regex = r'^( +|.|\n)$'
longer_than_70_regex = r'^(.{70,})$'
smaller_than_2_regex = r'^.{,2}$'
empty_line_regex = r'^(\s*)$'
trailing_spaces_and_tabs_regex = r'[ \t]+$'



from datetime import datetime
import regex

from beancount.core import data, flags
from beancount.core.amount import Amount
from beancount.core.number import Decimal
from beancount.ingest import importer

from .extract_statement import extractTextStatement
from .regex_formatter import *


class CEImporter_PDF(importer.ImporterProtocol):
    """Beancount Importer for Caisse d'Epargne PDF statement exports.

    Attributes:
        iban (str): International Bank Account Number of the account you want to extract operations. Note that only the account number is necessary
        account (str): Account name in beancount format (e.g. 'Assets:FR:CdE:CompteCourant')
        expenseCat (str, optional): Expense category in beancount format (e.g. 'Expenses:FIXME'). Defaults to '', no expense posting added to the operation.
        creditCat (str, optional): Income category in beancount format (e.g. 'Income:FIXME'). Defaults to '', no income posting added to the operation.
        showOperationTypes (bool, optional): Show or not operation type (CARDDEBIT, WIRETRANSFER, CHECK ...) in header. Defaults to False.
    """

    def __init__(
        self,
        iban: str,
        account: str,
        expenseCat: str = '',
        creditCat: str = '',
        showOperationTypes: bool = False,
    ):
        self.iban = iban
        self.account = account
        self.expenseCat = expenseCat
        self.creditCat = creditCat
        self.showOperationTypes = showOperationTypes

    ## API Methods ##
    #################

    def name(self):
        return 'CaisseEpargne {}'.format(self.__class__.__name__)

    def file_account(self, _):
        return self.account

    def file_date(self, file_):
        if not self.identify(file_):
            return None
        text = extractTextStatement(file_.name)
        return self._searchEmissionDate(text)

    def file_name(self, _):
        return 'CaisseEpargne_Statement.pdf'

    def identify(self, file_) -> bool:
        b = False
        try:
            text = extractTextStatement(file_.name)
            if 'www.caisse-epargne.fr' in text:
                b = True
        except:
            pass
        return b

    def extract(self, file_, existing_entries=None):
        entries = []

        if not self.identify(file_):
            return []

        if type(file_) == str:
            text = extractTextStatement(file_)
        else:
            text = extractTextStatement(file_.name)

        operations = self._getOperations(text)

        for index, op in enumerate(operations):
            isExpense = len(op[6]) > 0
            if type(file_) == str:
                meta = data.new_metadata(file_, index)
            else:
                meta = data.new_metadata(file_.name, index)

            amount = Decimal(op[6]) if isExpense else Decimal(op[5])
            currency = 'EUR'
            payee = op[4]
            if self.showOperationTypes:
                payee += ' - ' + op[2]
            postings = [
                data.Posting(
                    self.account,
                    Amount((-amount if isExpense else amount), currency),
                    None,
                    None,
                    None,
                    None,
                )
            ]
            if isExpense and len(self.expenseCat) > 0:
                postings.append(
                    data.Posting(
                        self.expenseCat,
                        Amount((amount if isExpense else -amount), currency),
                        None,
                        None,
                        None,
                        None,
                    )
                )

            if not isExpense and len(self.creditCat) > 0:
                postings.append(
                    data.Posting(
                        self.creditCat,
                        Amount((amount if isExpense else -amount), currency),
                        None,
                        None,
                        None,
                        None,
                    )
                )

            entries.append(
                data.Transaction(
                    meta,
                    datetime.strptime(op[0], '%d/%m/%Y').date(),
                    flags.FLAG_OKAY,
                    payee,
                    '',
                    data.EMPTY_SET,
                    data.EMPTY_SET,
                    postings,
                )
            )

        return entries

    ## Utils ##
    ###########

    def _getOperations(self, parsed_statement):
        # Clean-up statement string
        statement = regex.sub(
            one_character_line_regex,
            'FLAG_DELETE_THIS_LINE',
            parsed_statement,
            flags=regex.M,
        )  # flag lines with one character or less
        statement = '\n'.join(
            [
                s
                for s in statement.splitlines()
                if 'FLAG_DELETE_THIS_LINE' not in s
            ]
        )  # keep only non-flaged lines

        # Get emission date
        emission_date = regex.search(emission_date_regex, statement)
        emission_date = emission_date.group('date').strip()
        emission_date = datetime.strptime(emission_date, '%d/%m/%Y')

        accounts = self._searchAccounts(statement)

        operations = []

        for (full, account_number) in reversed(accounts):
            if account_number.replace(' ', '') in self.iban.replace(' ', ''):

                (statement, _, account) = statement.partition(full)

                # create total for inconsistency check
                total = Decimal(0.0)

                # clean account to keep only operations
                account = self._clean_account(account, full)
                # search all debit operations
                debit_ops = regex.finditer(debit_regex, account, flags=regex.M)
                for debit_op in debit_ops:
                    debitLine = account[debit_op.start() : debit_op.end()]
                    debitLine = debitLine.split('\n')[0]
                    debitLineList = debitLine.split(' ')[1:-1]
                    debitLineCleanList = []
                    for w in debitLineList:
                        if w == 'FACT':
                            break
                        if len(w) > 0:
                            debitLineCleanList.append(w)
                    debitLine = ''
                    for i, w in enumerate(debitLineCleanList):
                        debitLine += ('' if i == 0 else ' ') + w
                    # extract regex groups
                    op_date = debit_op.group('op_dte').strip()
                    op_label = debit_op.group('op_lbl').strip()
                    op_label_extra = debit_op.group('op_lbl_extra').strip()
                    op_amount = debit_op.group('op_amt').strip()
                    # convert amount to regular Decimal
                    op_amount = op_amount.replace(',', '.')
                    op_amount = op_amount.replace(' ', '')
                    op_amount = Decimal(op_amount)
                    # update total
                    total -= op_amount
                    # print('debit {0}'.format(op_amount))
                    operations.append(
                        self._create_operation_entry(
                            op_date,
                            emission_date,
                            full,
                            op_label,
                            debitLine,
                            op_amount,
                            True,
                        )
                    )

                # search all credit operations
                credit_ops = regex.finditer(
                    credit_regex, account, flags=regex.M
                )
                for credit_op in credit_ops:
                    # extract regex groups
                    op_date = credit_op.group('op_dte').strip()
                    op_label = credit_op.group('op_lbl').strip()
                    op_label_extra = credit_op.group('op_lbl_extra').strip()
                    op_amount = credit_op.group('op_amt').strip()

                    creditLine = account[credit_op.start() : credit_op.end()]
                    creditLine = creditLine.split('\n')[0]
                    creditLine = creditLine[len(op_amount) + len(op_date) :]

                    # convert amount to regular Decimal
                    op_amount = op_amount.replace(',', '.')
                    op_amount = op_amount.replace(' ', '')
                    op_amount = Decimal(op_amount)
                    # update total
                    total += op_amount
                    # print('credit {0}'.format(op_amount))

                    operations.append(
                        self._create_operation_entry(
                            op_date,
                            emission_date,
                            full,
                            op_label,
                            creditLine,
                            op_amount,
                            False,
                        )
                    )

        return operations

    def _create_operation_entry(
        self,
        op_date,
        statement_emission_date,
        account_number,
        op_label,
        op_label_extra,
        op_amount,
        debit,
    ):
        # search the operation type according to its label
        op_type = self._search_operation_type(op_label)

        op = [
            self._set_operation_year(op_date, statement_emission_date),
            account_number,
            op_type,
            op_label.strip(),
            # op_label_extra.strip().replace('\n','\\'),
            op_label_extra.strip(),
            # the star '*' operator is like spread '...' in JS
            *(['', '%.2f' % op_amount] if debit else ['%.2f' % op_amount, '']),
        ]
        return op

    def _search_operation_type(self, op_label):
        op_label = op_label.upper()
        # bank fees, international fees, subscription fee to bouquet, etc.
        if (op_label.startswith('*')) or (op_label.startswith('INTERETS')):
            opType = 'BANK'
        # cash deposits on the account
        elif op_label.startswith('VERSEMENT'):
            opType = 'DEPOSIT'
        # incoming / outcoming wire transfers: salary, p2p, etc.
        elif (op_label.startswith('VIREMENT')) or (op_label.startswith('VIR')):
            opType = 'WIRETRANSFER'
        # check deposits / payments
        elif (
            (op_label.startswith('CHEQUE'))
            or (op_label.startswith('REMISE CHEQUES'))
            or (op_label.startswith('REMISE CHQ'))
        ):
            opType = 'CHECK'
        # payments made via debit card
        elif op_label.startswith('CB'):
            opType = 'CARDDEBIT'
        # withdrawals
        elif (op_label.startswith('RETRAIT')) or (
            op_label.startswith('RET DAB')
        ):
            opType = 'WITHDRAWAL'
        # direct debits
        elif op_label.startswith('PRLV'):
            opType = 'DIRECTDEBIT'
        else:
            opType = 'OTHER'

        return opType

    def _set_operation_year(self, emission, statement_emission_date):
        # fake a leap year
        emission = datetime.strptime(emission + '00', '%d/%m%y')
        if emission.month <= statement_emission_date.month:
            emission = emission.replace(year=statement_emission_date.year)
        else:
            emission = emission.replace(year=statement_emission_date.year - 1)
        return datetime.strftime(emission, '%d/%m/%Y')

    def _clean_account(self, account, account_number):
        # split the text by the 'new_balance_regex' line
        cleaned = regex.split(new_balance_regex, account, flags=regex.M)
        # keep the first part (i.e. everything that's before the 'new_balance_regex' line)
        cleaned = cleaned[0]
        # flag lines with specific words
        words_to_remove = [
            account_number,
            'Relevé',
            'vos comptes',
            'Page',
            'Débit Crédit',
            'Détail des opérations',
            'frais bancaires et cotisations',
            'SOLDE PRECEDENT AU',
        ]
        words_to_remove_regex = (
            r'^.*\b(' + '|'.join(words_to_remove) + r')\b.*$'
        )
        # flag lines longer than 70
        cleaned = regex.sub(
            longer_than_70_regex,
            'FLAG_DELETE_THIS_LINE',
            cleaned,
            flags=regex.M,
        )
        # flag lines with words to remove
        cleaned = regex.sub(
            words_to_remove_regex,
            'FLAG_DELETE_THIS_LINE',
            cleaned,
            flags=regex.M,
        )
        # remove trailing spaces
        cleaned = regex.sub(
            trailing_spaces_and_tabs_regex, '', cleaned, flags=regex.M
        )
        # flag empty lines
        cleaned = regex.sub(
            empty_line_regex, 'FLAG_DELETE_THIS_LINE', cleaned, flags=regex.M
        )
        # flag lines with less than 2 characters
        cleaned = regex.sub(
            smaller_than_2_regex,
            'FLAG_DELETE_THIS_LINE',
            cleaned,
            flags=regex.M,
        )
        # keep only non-flaged lines
        cleaned = '\n'.join(
            [
                s
                for s in cleaned.splitlines()
                if 'FLAG_DELETE_THIS_LINE' not in s
            ]
        )
        return cleaned

    def _search_account_owner(self, regex_to_use, statement):
        # search for owner to identify multiple accounts
        account_owner = regex.search(regex_to_use, statement, flags=regex.M)
        if not account_owner:
            raise ValueError('No account owner was found.')
        # extract and strip
        account_owner = account_owner.group('owner').strip()
        return account_owner

    def _searchEmissionDate(self, statement):
        emission_date = regex.search(emission_date_regex, statement)
        # extract and strip
        emission_date = emission_date.group('date').strip()
        # parse date
        emission_date = datetime.strptime(emission_date, '%d/%m/%Y')
        return emission_date.date()

    def _searchAccounts(self, statement):
        # get owner
        owner = self._search_account_owner(owner_regex_v1, statement)

        account_regex = r'^((?:MR|MME|MLLE) ' + owner + ' - .* - ([^(\n]*))$'
        accounts = regex.findall(account_regex, statement, flags=regex.M)

        # no accounts found, try to get owner with other regex
        if len(accounts) == 0:
            owner = self._search_account_owner(owner_regex_v2, statement)
            account_regex = (
                r'^((?:MR|MME|MLLE) ' + owner + ' - .* - ([^(\n]*))$'
            )
            accounts = regex.findall(account_regex, statement, flags=regex.M)

        # cleanup account number for each returned account
        # we use a syntax called 'list comprehension'
        cleaned_accounts = [
            (full, regex.sub(r'\D', '', account_number))
            for (full, account_number) in accounts
        ]
        return cleaned_accounts



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['beancount_ce']

package_data = \
{'': ['*']}

install_requires = \
['beancount>=2.2,<3.0',
 'pdfminer.six>=20200726,<20200727',
 'regex>=2020.7.14,<2021.0.0']

setup_kwargs = {
    'name': 'beancount-ce',
    'version': '1.0.5',
    'description': "Beancount statements (pdf and csv) importer for Caisse d'Epargne bank",
    'long_description': "# Beancount Caisse d'Epargne Importer\n\n[![GitHub Workflow Status](https://img.shields.io/github/workflow/status/ArthurFDLR/beancount-ce/beancount-ce?style=for-the-badge)](https://github.com/ArthurFDLR/beancount-ce/actions)\n[![PyPI](https://img.shields.io/pypi/v/beancount-ce?style=for-the-badge)](https://pypi.org/project/beancount-ce/)\n[![PyPI - Version](https://img.shields.io/pypi/pyversions/beancount-ce.svg?style=for-the-badge)](https://pypi.org/project/beancount-ce/)\n[![GitHub](https://img.shields.io/github/license/ArthurFDLR/beancount-ce?style=for-the-badge)](https://github.com/ArthurFDLR/beancount-ce/blob/master/LICENSE.txt)\n[![Linting](https://img.shields.io/badge/code%20style-black-000000.svg?style=for-the-badge)](https://github.com/psf/black)\n\n`beancount-ce` provides a statements (PDF and CSV) importer for the bank [Caisse d'Epargne](http://www.caisse-epargne.fr) to the [Beancount](http://furius.ca/beancount/) format.\n\n## Installation\n\n```console\n    $ pip install beancount-ce\n```\n\n## Usage\n\nAdd ```CEImporter``` to your [Beancount importers config file](https://beancount.github.io/docs/importing_external_data.html#configuration).\n\n```python\n    IBAN_NUMBER_CE = 'FR00 1111 2222 3333 4444 5555 666'\n\n    CONFIG = [\n        CEImporter(\n            iban=IBAN_NUMBER_CE,\n            account='Assets:FR:CdE:CompteCourant',\n            expenseCat='Expenses:FIXME',    #Optional\n            creditCat='Income:FIXME',       #Optional\n            showOperationTypes=False        #Optional\n        ),\n    ]\n```\n\n## Contribution\n\nFeel free to contribute!\n\nPlease make sure you have Python 3.6+ and [`Poetry`](https://poetry.eustace.io/) installed.\n\n1. Git clone the repository - `git clone https://github.com/ArthurFDLR/beancount-ce`\n\n2. Install the packages required for development - `poetry install`\n\n3. That's basically it. You should now be able to run lint checks and the test suite - `make lint test`.\n",
    'author': 'Arthur Findelair',
    'author_email': 'arthfind@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ArthurFDLR/beancount-ce',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from pdfminer.high_level import extract_text
from pdfminer.layout import LAParams


def extractTextStatement(pdf_file):
    if pdf_file.split('.')[-1] == 'txt':
        with open(pdf_file, 'r') as f:
            data = f.read()
        return str(data)

    elif pdf_file.split('.')[-1] == 'pdf':
        word_margin = 1.0
        char_margin = 120.0
        line_margin = 0.3
        boxes_flow = 0.5  # LAParams

        # If any LAParams group arguments were passed, create an LAParams object and
        # populate with given args. Otherwise, set it to None.

        laparams = LAParams()
        for param in (
            "word_margin",
            "char_margin",
            "line_margin",
            "boxes_flow",
        ):
            paramv = locals().get(param, None)
            if paramv is not None:
                setattr(laparams, param, paramv)

        text = extract_text(pdf_file=pdf_file, laparams=laparams)
        return text


if __name__ == "__main__":
    import sys

    file_name = ''
    for arg in sys.argv:
        if arg[-4:] == '.pdf':
            file_name = arg

    if len(file_name) > 0:
        print('Extract text from ' + file_name)
        with open(file_name[:-4] + '.txt', 'w', encoding='utf8') as text_file:
            statement = extractTextStatement(str(file_name))
            text_file.write(statement)
    else:
        print('No PDF file found.')
